
# q-expansion of newform 3520.2.a.w, downloaded from the LMFDB on 09 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3520
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [-1], [1], [1], [2], [-5], [-7], [6], [1], [5], [-11], [2], [4], [-6], [1], [-10], [-5], [-8], [-9], [-6], [10], [-6], [17], [16], [2], [-12], [-12], [-10], [-12], [8], [-19], [8], [4], [3], [6], [-3], [5], [-23], [2], [-24], [14], [-20], [-17], [8], [15], [-27], [10], [14], [14], [3], [18], [26], [-2], [-14], [-21], [-24], [-20], [24], [-2], [18], [10], [6], [-11], [22], [-9], [-12], [-3], [10], [22], [-2], [-8], [32], [-22], [-22], [-34], [-6], [-22], [-5], [-12], [-4], [32], [-12], [0], [20], [36], [26], [-15], [15], [10], [9], [12], [-4], [-7], [-28], [24], [12], [-18], [32], [45], [-20], [14], [14], [-36], [31], [14], [19], [18], [-11], [30], [43], [-2], [24], [12], [-1], [-25], [-11], [-2], [-49], [-5], [-22], [-33], [32], [47], [-10], [-5], [-42], [-31], [-26], [24], [-32], [9], [41], [10], [-2], [-10], [29], [-4], [-2], [-42], [19], [-18], [12], [-24], [16], [24], [34], [51], [-2], [-6], [58], [-14], [-41], [8], [-23], [19], [16], [19], [10], [53], [-13], [29], [-1], [-32], [32], [6], [48], [-32], [2], [-19], [57], [-42], [-18], [39], [-8], [-61], [-52], [16], [43], [63], [18], [-19], [-29], [-41], [22], [-30], [-12], [-28], [14], [-24], [8], [8], [-34], [-44], [-40], [-42], [20], [30], [22], [35], [-39], [-10], [11], [-8], [8], [-3], [-56], [-16], [50], [-8], [33], [0], [48], [-45], [-32], [19], [51], [26], [-64], [35], [-58], [-46], [45], [-20], [72], [59], [-54], [-69], [-64], [60], [30], [-61], [-50], [-38], [36], [-45], [-53], [-6], [-56], [65], [-29], [-4], [32], [-18], [-66], [-54], [0], [-30], [37], [11], [22], [-60], [-18], [-29], [76], [-27], [-24], [-18], [-6], [37], [-24], [12], [15], [-62], [0], [28], [-8], [9], [-64], [-54], [30], [-17], [40], [-82], [43], [-46], [-48], [26], [-70], [-33], [-31], [-44], [-56], [83], [56], [21], [2], [42], [-37], [-18], [-4], [-60], [-17], [11], [26], [-78], [-56], [-21], [4], [62], [49], [-60], [-24], [-36], [-50], [-64], [-4], [-40], [9], [-56], [-84], [-90], [88], [45], [17], [-58], [38], [84], [-78], [60], [3], [-38], [10], [14], [5], [-21], [-38], [-44], [-83], [44], [-35], [62], [-44], [-10], [-23], [-71], [-2], [-21], [77], [53], [54], [54], [69], [-49], [-39], [63], [28], [-53], [-24], [7], [85], [-2], [60], [-61], [-53], [-75], [-71], [-69], [-6], [48], [10], [68], [68], [-29], [-43], [72], [-46], [53], [36], [-20], [56], [0], [75], [62], [-22], [0], [2], [59], [28], [-47], [-28], [82], [20], [-67], [-56], [80], [16], [-5], [-40], [-54], [-35], [47], [12], [72], [24], [54], [-6], [7], [34], [51], [69], [-14], [28], [0], [-90], [38], [-25], [-82], [47], [70], [14], [-8], [-36], [28], [50], [64], [-40], [92], [-30], [-35], [-96], [64], [-24], [33], [-10], [-57], [11], [20], [52], [90]]
